/*
 *
 * Copyright (c) 2003-2004 Erez Zadok
 * Copyright (c) 2003-2004 Swapnil V. Patil
 * Copyright (c) 2003-2004 Anand Kashyap
 * Copyright (c) 2003-2004 Gopalan Sivathanu
 * Copyright (c) 2003-2004 Stony Brook University
 * Copyright (c) 2003-2004 The Research Foundation of State University of New York
 *
 * For specific licensing information, see the COPYING file distributed with
 * this package.
 *
 * This Copyright notice must be kept intact and distributed with all sources.
 */


/*
 *  $Id: fist_ioctl.c,v 1.2 2004/06/07 20:48:33 gopalan Exp $
 */

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif /* HAVE_CONFIG_H */
#ifdef FISTGEN
# include "fist_i3fs.h"
#endif /* FISTGEN */
#include "fist.h"
#include "i3fs.h"

#include <sys/ioctl.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

static int opt_d = 0;
static int opt_c = 0;
static int optcount;
static const char *progname;


void
usage(void)
{
    fprintf(stderr, "Usage: %s -d file [val]\n\tto set/get debugging values\n", progname);
    exit(1);
}


int
main(int argc, char *argv[])
{
    int fd, ret, val = 0;
    long checksum = 0;

    progname = argv[0];

    /* check that minimum number of args were specified */
    if (argc < 3)
	usage();

    if (strcmp(argv[1], "-d") == 0) {
	if (argc > 4)
	    usage();
	opt_d++;
	optcount++;
    }

    if (strcmp(argv[1], "-c") == 0) {
	if (argc > 4)
	    usage();
	opt_c++;
	optcount++;
    }
    
    /* check that at least one option was used */
    if (!optcount)
	usage();

    /* open file on which ioctl will operate */
    fd = open(argv[2], O_RDONLY);
    if (fd < 0) {
	perror(argv[2]);
	exit(1);
    }

    /* if specified 3rd arg, want to set debug level */
    if (opt_d) {
	if (argc == 4) {
	    val = atoi(argv[3]);
	    ret = ioctl(fd, FIST_IOCTL_SET_DEBUG_VALUE, &val);
	    if (ret < 0) {
		perror("ioctl set");
		exit(1);
	    }
	} else {
	    ret = ioctl(fd, FIST_IOCTL_GET_DEBUG_VALUE, &val);
	    if (ret < 0) {
		perror("ioctl get");
		exit(1);
	    }
	    printf("debug ioctl returned value %d\n", val);
	}
	goto out;
    }
    if (opt_c) {
	ret = ioctl(fd, FIST_IOCTL_GET_FILE_CHECKSUM, &checksum);
	if (ret < 0) {
		perror("ioctl get");
		exit(1);
    	}
	printf("File checksum returned %ld\n", checksum);
    }

 out:
    close(fd);
    exit(0);
}

/*
 * Local variables:
 * c-basic-offset: 4
 * End:
 */
